<?php
/**
 * @package    GetBible
 *
 * @created    30th May, 2023
 * @author     Llewellyn van der Merwe <https://dev.vdm.io>
 * @git        GetBible <https://git.vdm.dev/getBible>
 * @copyright  Copyright (C) 2015 Vast Development Method. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace TrueChristianChurch\Joomla\GetBible;


use TrueChristianChurch\Joomla\Interfaces\Tableinterface;
use TrueChristianChurch\Joomla\Abstraction\BaseTable;


/**
 * GetBible Tables
 * 
 * @since 2.0.1
 */
class Table extends BaseTable implements Tableinterface
{
	/**
	 * All areas/views/tables with their field details
	 *
	 * @var     array
	 * @since  2.0.1
	 **/
	protected array $tables = [
		'linker' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_LINKER_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'linkers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_LINKER_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'linkers',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'public_tagged_verses' => [
				'name' => 'public_tagged_verses',
				'label' => 'COM_GETBIBLE_LINKER_PUBLIC_TAGGED_VERSES_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'linkers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'public_notes' => [
				'name' => 'public_notes',
				'label' => 'COM_GETBIBLE_LINKER_PUBLIC_NOTES_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'linkers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'note' => [
			'book_nr' => [
				'name' => 'book_nr',
				'label' => 'COM_GETBIBLE_NOTE_BOOK_NR_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'linker' => [
				'name' => 'linker',
				'label' => 'COM_GETBIBLE_NOTE_LINKER_LABEL',
				'type' => 'linkers',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'COM_GETBIBLE_NOTE_ACCESS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_NOTE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'note' => [
				'name' => 'note',
				'label' => 'COM_GETBIBLE_NOTE_NOTE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'verse' => [
				'name' => 'verse',
				'label' => 'COM_GETBIBLE_NOTE_VERSE_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'chapter' => [
				'name' => 'chapter',
				'label' => 'COM_GETBIBLE_NOTE_CHAPTER_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'notes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
		],
		'tagged_verse' => [
			'book_nr' => [
				'name' => 'book_nr',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_BOOK_NR_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_ABBREVIATION_LABEL',
				'type' => 'translations',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_ACCESS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'linker' => [
				'name' => 'linker',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_LINKER_LABEL',
				'type' => 'linkers',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'tag' => [
				'name' => 'tag',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_TAG_LABEL',
				'type' => 'tagers',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'verse' => [
				'name' => 'verse',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_VERSE_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'chapter' => [
				'name' => 'chapter',
				'label' => 'COM_GETBIBLE_TAGGED_VERSE_CHAPTER_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'tagged_verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
		],
		'prompt' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_PROMPT_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'integration' => [
				'name' => 'integration',
				'label' => 'COM_GETBIBLE_PROMPT_INTEGRATION_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'cache_behaviour' => [
				'name' => 'cache_behaviour',
				'label' => 'COM_GETBIBLE_PROMPT_CACHE_BEHAVIOUR_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_PROMPT_ABBREVIATION_LABEL',
				'type' => 'targettranslations',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_PROMPT_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'model' => [
				'name' => 'model',
				'label' => 'COM_GETBIBLE_PROMPT_MODEL_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'VARCHAR(50)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'presence_penalty' => [
				'name' => 'presence_penalty',
				'label' => 'COM_GETBIBLE_PROMPT_PRESENCE_PENALTY_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'org_token' => [
				'name' => 'org_token',
				'label' => 'COM_GETBIBLE_PROMPT_ORG_TOKEN_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'token' => [
				'name' => 'token',
				'label' => 'COM_GETBIBLE_PROMPT_TOKEN_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'n_override' => [
				'name' => 'n_override',
				'label' => 'COM_GETBIBLE_PROMPT_N_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'cache_capacity' => [
				'name' => 'cache_capacity',
				'label' => 'COM_GETBIBLE_PROMPT_CACHE_CAPACITY_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'messages' => [
				'name' => 'messages',
				'label' => 'COM_GETBIBLE_PROMPT_MESSAGES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'prompts',
				'store' => 'json',
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'response_retrieval' => [
				'name' => 'response_retrieval',
				'label' => 'COM_GETBIBLE_PROMPT_RESPONSE_RETRIEVAL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'frequency_penalty_override' => [
				'name' => 'frequency_penalty_override',
				'label' => 'COM_GETBIBLE_PROMPT_FREQUENCY_PENALTY_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'n' => [
				'name' => 'n',
				'label' => 'COM_GETBIBLE_PROMPT_N_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'max_tokens_override' => [
				'name' => 'max_tokens_override',
				'label' => 'COM_GETBIBLE_PROMPT_MAX_TOKENS_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'token_override' => [
				'name' => 'token_override',
				'label' => 'COM_GETBIBLE_PROMPT_TOKEN_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'max_tokens' => [
				'name' => 'max_tokens',
				'label' => 'COM_GETBIBLE_PROMPT_MAX_TOKENS_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'ai_org_token_override' => [
				'name' => 'ai_org_token_override',
				'label' => 'COM_GETBIBLE_PROMPT_AI_ORG_TOKEN_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'temperature_override' => [
				'name' => 'temperature_override',
				'label' => 'COM_GETBIBLE_PROMPT_TEMPERATURE_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'presence_penalty_override' => [
				'name' => 'presence_penalty_override',
				'label' => 'COM_GETBIBLE_PROMPT_PRESENCE_PENALTY_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'top_p_override' => [
				'name' => 'top_p_override',
				'label' => 'COM_GETBIBLE_PROMPT_TOP_P_OVERRIDE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'frequency_penalty' => [
				'name' => 'frequency_penalty',
				'label' => 'COM_GETBIBLE_PROMPT_FREQUENCY_PENALTY_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'top_p' => [
				'name' => 'top_p',
				'label' => 'COM_GETBIBLE_PROMPT_TOP_P_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'temperature' => [
				'name' => 'temperature',
				'label' => 'COM_GETBIBLE_PROMPT_TEMPERATURE_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'prompts',
				'store' => NULL,
				'tab_name' => 'Open AI',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'open_ai_response' => [
			'response_id' => [
				'name' => 'response_id',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_RESPONSE_ID_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'prompt' => [
				'name' => 'prompt',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_PROMPT_LABEL',
				'type' => 'prompts',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'response_object' => [
				'name' => 'response_object',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_RESPONSE_OBJECT_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'response_model' => [
				'name' => 'response_model',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_RESPONSE_MODEL_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'total_tokens' => [
				'name' => 'total_tokens',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_TOTAL_TOKENS_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'n' => [
				'name' => 'n',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_N_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'frequency_penalty' => [
				'name' => 'frequency_penalty',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_FREQUENCY_PENALTY_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'presence_penalty' => [
				'name' => 'presence_penalty',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_PRESENCE_PENALTY_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'word' => [
				'name' => 'word',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_WORD_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'chapter' => [
				'name' => 'chapter',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_CHAPTER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'lcsh' => [
				'name' => 'lcsh',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_LCSH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'completion_tokens' => [
				'name' => 'completion_tokens',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_COMPLETION_TOKENS_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'prompt_tokens' => [
				'name' => 'prompt_tokens',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_PROMPT_TOKENS_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'response_created' => [
				'name' => 'response_created',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_RESPONSE_CREATED_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_ABBREVIATION_LABEL',
				'type' => 'targettranslations',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'language' => [
				'name' => 'language',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_LANGUAGE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'max_tokens' => [
				'name' => 'max_tokens',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_MAX_TOKENS_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'book' => [
				'name' => 'book',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_BOOK_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'temperature' => [
				'name' => 'temperature',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_TEMPERATURE_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'verse' => [
				'name' => 'verse',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_VERSE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'top_p' => [
				'name' => 'top_p',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_TOP_P_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'FLOAT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'selected_word' => [
				'name' => 'selected_word',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_SELECTED_WORD_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Bible',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'model' => [
				'name' => 'model',
				'label' => 'COM_GETBIBLE_OPEN_AI_RESPONSE_MODEL_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'open_ai_responses',
				'store' => NULL,
				'tab_name' => 'Prompt',
				'db' => [
					'type' => 'VARCHAR(50)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'open_ai_message' => [
			'role' => [
				'name' => 'role',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_ROLE_LABEL',
				'type' => 'list',
				'title' => true,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'open_ai_response' => [
				'name' => 'open_ai_response',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_OPEN_AI_RESPONSE_LABEL',
				'type' => 'openairesponses',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'prompt' => [
				'name' => 'prompt',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_PROMPT_LABEL',
				'type' => 'prompts',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'source' => [
				'name' => 'source',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_SOURCE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'content' => [
				'name' => 'content',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_CONTENT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'index' => [
				'name' => 'index',
				'label' => 'COM_GETBIBLE_OPEN_AI_MESSAGE_INDEX_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'open_ai_messages',
				'store' => NULL,
				'tab_name' => 'Message',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'password' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_PASSWORD_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'passwords',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'linker' => [
				'name' => 'linker',
				'label' => 'COM_GETBIBLE_PASSWORD_LINKER_LABEL',
				'type' => 'linkers',
				'title' => false,
				'list' => 'passwords',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_PASSWORD_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'passwords',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'password' => [
				'name' => 'password',
				'label' => 'COM_GETBIBLE_PASSWORD_PASSWORD_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'passwords',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'tag' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_TAG_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'tags',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'linker' => [
				'name' => 'linker',
				'label' => 'COM_GETBIBLE_TAG_LINKER_LABEL',
				'type' => 'linkers',
				'title' => false,
				'list' => 'tags',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'COM_GETBIBLE_TAG_ACCESS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'tags',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_GETBIBLE_TAG_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'tags',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_GETBIBLE_TAG_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'tags',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
		],
		'translation' => [
			'translation' => [
				'name' => 'translation',
				'label' => 'COM_GETBIBLE_TRANSLATION_TRANSLATION_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_TRANSLATION_ABBREVIATION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'language' => [
				'name' => 'language',
				'label' => 'COM_GETBIBLE_TRANSLATION_LANGUAGE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'direction' => [
				'name' => 'direction',
				'label' => 'COM_GETBIBLE_TRANSLATION_DIRECTION_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_history' => [
				'name' => 'distribution_history',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_HISTORY_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'translations',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_about' => [
				'name' => 'distribution_about',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_ABOUT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_license' => [
				'name' => 'distribution_license',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_LICENSE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_source' => [
				'name' => 'distribution_source',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_SOURCE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_sourcetype' => [
				'name' => 'distribution_sourcetype',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_SOURCETYPE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_versification' => [
				'name' => 'distribution_versification',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_VERSIFICATION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'sha' => [
				'name' => 'sha',
				'label' => 'COM_GETBIBLE_TRANSLATION_SHA_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'encoding' => [
				'name' => 'encoding',
				'label' => 'COM_GETBIBLE_TRANSLATION_ENCODING_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_lcsh' => [
				'name' => 'distribution_lcsh',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_LCSH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_version_date' => [
				'name' => 'distribution_version_date',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_VERSION_DATE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_version' => [
				'name' => 'distribution_version',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_VERSION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'lang' => [
				'name' => 'lang',
				'label' => 'COM_GETBIBLE_TRANSLATION_LANG_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'distribution_abbreviation' => [
				'name' => 'distribution_abbreviation',
				'label' => 'COM_GETBIBLE_TRANSLATION_DISTRIBUTION_ABBREVIATION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'book' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_BOOK_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'books',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_BOOK_ABBREVIATION_LABEL',
				'type' => 'translations',
				'title' => false,
				'list' => 'books',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'nr' => [
				'name' => 'nr',
				'label' => 'COM_GETBIBLE_BOOK_NR_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'books',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'sha' => [
				'name' => 'sha',
				'label' => 'COM_GETBIBLE_BOOK_SHA_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'books',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'chapter' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_CHAPTER_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'chapters',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'chapter' => [
				'name' => 'chapter',
				'label' => 'COM_GETBIBLE_CHAPTER_CHAPTER_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'chapters',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'book_nr' => [
				'name' => 'book_nr',
				'label' => 'COM_GETBIBLE_CHAPTER_BOOK_NR_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'chapters',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_CHAPTER_ABBREVIATION_LABEL',
				'type' => 'translations',
				'title' => false,
				'list' => 'chapters',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'sha' => [
				'name' => 'sha',
				'label' => 'COM_GETBIBLE_CHAPTER_SHA_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'chapters',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
		'verse' => [
			'book_nr' => [
				'name' => 'book_nr',
				'label' => 'COM_GETBIBLE_VERSE_BOOK_NR_LABEL',
				'type' => 'number',
				'title' => true,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'chapter' => [
				'name' => 'chapter',
				'label' => 'COM_GETBIBLE_VERSE_CHAPTER_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'verse' => [
				'name' => 'verse',
				'label' => 'COM_GETBIBLE_VERSE_VERSE_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(7)',
					'default' => '0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'abbreviation' => [
				'name' => 'abbreviation',
				'label' => 'COM_GETBIBLE_VERSE_ABBREVIATION_LABEL',
				'type' => 'translations',
				'title' => false,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'text' => [
				'name' => 'text',
				'label' => 'COM_GETBIBLE_VERSE_TEXT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_GETBIBLE_VERSE_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'verses',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NOT NULL',
				],
			],
		],
	];
}

